==> ginkgo-1.4.0.tar.gz: ginkgo-1.4.0/reference/test/log/convergence.cpp <==
/*******************************<GINKGO LICENSE>******************************
Copyright (c) 2017-2021, the Ginkgo authors
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
******************************<GINKGO LICENSE>*******************************/

#include <ginkgo/core/log/convergence.hpp>


#include <gtest/gtest.h>


#include <ginkgo/core/base/executor.hpp>
#include <ginkgo/core/base/math.hpp>
#include <ginkgo/core/matrix/dense.hpp>
#include <ginkgo/core/stop/iteration.hpp>


#include "core/test/utils.hpp"


namespace {


template <typename T>
class Convergence : public ::testing::Test {};

TYPED_TEST_SUITE(Convergence, gko::test::ValueTypes);


TYPED_TEST(Convergence, CatchesCriterionCheckCompleted)
{
    auto exec = gko::ReferenceExecutor::create();
    auto logger = gko::log::Convergence<TypeParam>::create(
        exec, gko::log::Logger::criterion_check_completed_mask);
    auto criterion =
        gko::stop::Iteration::build().with_max_iters(3u).on(exec)->generate(
            nullptr, nullptr, nullptr);
    constexpr gko::uint8 RelativeStoppingId{42};
    gko::Array<gko::stopping_status> stop_status(exec, 1);
    stop_status.get_data()[0].reset();
    using Mtx = gko::matrix::Dense<TypeParam>;
    using NormVector = gko::matrix::Dense<gko::remove_complex<TypeParam>>;
    auto residual = gko::initialize<Mtx>({1.0, 2.0, 2.0}, exec);

    logger->template on<gko::log::Logger::criterion_check_completed>(
        criterion.get(), 1, residual.get(), nullptr, nullptr,
        RelativeStoppingId, true, &stop_status, true, true);

    ASSERT_EQ(logger->get_num_iterations(), 1);
    ASSERT_EQ(logger->has_converged(), false);
    GKO_ASSERT_MTX_NEAR(gko::as<Mtx>(logger->get_residual()),
                        l({1.0, 2.0, 2.0}), 0.0);
    GKO_ASSERT_MTX_NEAR(gko::as<NormVector>(logger->get_residual_norm()),
                        l({3.0}), 0.0);
}


TYPED_TEST(Convergence, CatchesCriterionCheckCompletedWithConvCheck)
{
    auto exec = gko::ReferenceExecutor::create();
    auto logger = gko::log::Convergence<TypeParam>::create(
        exec, gko::log::Logger::criterion_check_completed_mask);
    auto criterion =
        gko::stop::Iteration::build().with_max_iters(3u).on(exec)->generate(
            nullptr, nullptr, nullptr);
    constexpr gko::uint8 RelativeStoppingId{42};
    gko::Array<gko::stopping_status> stop_status(exec, 1);
    stop_status.get_data()[0].reset();
    stop_status.get_data()[0].converge(0);
    using Mtx = gko::matrix::Dense<TypeParam>;
    using NormVector = gko::matrix::Dense<gko::remove_complex<TypeParam>>;
    auto residual = gko::initialize<Mtx>({1.0, 2.0, 2.0}, exec);

    logger->template on<gko::log::Logger::criterion_check_completed>(
        criterion.get(), 3, residual.get(), nullptr, nullptr,
        RelativeStoppingId, true, &stop_status, true, true);

    ASSERT_EQ(logger->get_num_iterations(), 3);
    ASSERT_EQ(logger->has_converged(), true);
    GKO_ASSERT_MTX_NEAR(gko::as<Mtx>(logger->get_residual()),
                        l({1.0, 2.0, 2.0}), 0.0);
    GKO_ASSERT_MTX_NEAR(gko::as<NormVector>(logger->get_residual_norm()),
                        l({3.0}), 0.0);
}


TYPED_TEST(Convergence, CatchesCriterionCheckCompletedWithStopCheck)
{
    auto exec = gko::ReferenceExecutor::create();
    auto logger = gko::log::Convergence<TypeParam>::create(
        exec, gko::log::Logger::criterion_check_completed_mask);
    auto criterion =
        gko::stop::Iteration::build().with_max_iters(3u).on(exec)->generate(
            nullptr, nullptr, nullptr);
    constexpr gko::uint8 RelativeStoppingId{42};
    gko::Array<gko::stopping_status> stop_status(exec, 1);
    stop_status.get_data()[0].reset();
    stop_status.get_data()[0].stop(0);
    using Mtx = gko::matrix::Dense<TypeParam>;
    using NormVector = gko::matrix::Dense<gko::remove_complex<TypeParam>>;
    auto residual = gko::initialize<Mtx>({1.0, 2.0, 2.0}, exec);

    logger->template on<gko::log::Logger::criterion_check_completed>(
        criterion.get(), 3, residual.get(), nullptr, nullptr,
        RelativeStoppingId, true, &stop_status, true, true);

    ASSERT_EQ(logger->get_num_iterations(), 3);
    ASSERT_EQ(logger->has_converged(), false);
    GKO_ASSERT_MTX_NEAR(gko::as<Mtx>(logger->get_residual()),
                        l({1.0, 2.0, 2.0}), 0.0);
    GKO_ASSERT_MTX_NEAR(gko::as<NormVector>(logger->get_residual_norm()),
                        l({3.0}), 0.0);
}


TYPED_TEST(Convergence, CanResetConvergenceStatus)
{
    auto exec = gko::ReferenceExecutor::create();
    auto logger = gko::log::Convergence<TypeParam>::create(
        exec, gko::log::Logger::criterion_check_completed_mask);
    auto criterion =
        gko::stop::Iteration::build().with_max_iters(3u).on(exec)->generate(
            nullptr, nullptr, nullptr);
    constexpr gko::uint8 RelativeStoppingId{42};
    gko::Array<gko::stopping_status> stop_status(exec, 1);
    stop_status.get_data()[0].reset();
    stop_status.get_data()[0].converge(0);
    using Mtx = gko::matrix::Dense<TypeParam>;
    using NormVector = gko::matrix::Dense<gko::remove_complex<TypeParam>>;
    auto residual = gko::initialize<Mtx>({1.0, 2.0, 2.0}, exec);

    logger->template on<gko::log::Logger::criterion_check_completed>(
        criterion.get(), 3, residual.get(), nullptr, nullptr,
        RelativeStoppingId, true, &stop_status, true, true);
    ASSERT_EQ(logger->get_num_iterations(), 3);
    ASSERT_EQ(logger->has_converged(), true);

    logger->reset_convergence_status();

    ASSERT_EQ(logger->has_converged(), false);
}


TYPED_TEST(Convergence, CatchesCriterionCheckCompletedWithImplicitNorm)
{
    auto exec = gko::ReferenceExecutor::create();
    auto logger = gko::log::Convergence<TypeParam>::create(
        exec, gko::log::Logger::criterion_check_completed_mask);
    auto criterion =
        gko::stop::Iteration::build().with_max_iters(3u).on(exec)->generate(
            nullptr, nullptr, nullptr);
    constexpr gko::uint8 RelativeStoppingId{42};
    gko::Array<gko::stopping_status> stop_status(exec, 1);
    stop_status.get_data()[0].reset();
    using Mtx = gko::matrix::Dense<TypeParam>;
    using NormVector = gko::matrix::Dense<gko::remove_complex<TypeParam>>;
    auto residual = gko::initialize<Mtx>({1.0, 2.0, 2.0}, exec);
    auto implicit_sq_resnorm = gko::initialize<Mtx>({4.0}, exec);

    logger->template on<gko::log::Logger::criterion_check_completed>(
        criterion.get(), 1, residual.get(), nullptr, implicit_sq_resnorm.get(),
        nullptr, RelativeStoppingId, true, &stop_status, true, true);

    ASSERT_EQ(logger->get_num_iterations(), 1);
    ASSERT_EQ(logger->has_converged(), false);
    GKO_ASSERT_MTX_NEAR(gko::as<Mtx>(logger->get_implicit_sq_resnorm()),
                        l({4.0}), 0.0);
    GKO_ASSERT_MTX_NEAR(gko::as<Mtx>(logger->get_residual()),
                        l({1.0, 2.0, 2.0}), 0.0);
    GKO_ASSERT_MTX_NEAR(gko::as<NormVector>(logger->get_residual_norm()),
                        l({3.0}), 0.0);
}


}  // namespace

==> ginkgo-1.6.0.tar.gz: ginkgo-1.6.0/reference/test/log/convergence.cpp <==
/*******************************<GINKGO LICENSE>******************************
Copyright (c) 2017-2023, the Ginkgo authors
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
******************************<GINKGO LICENSE>*******************************/

#include <ginkgo/core/log/convergence.hpp>


#include <gtest/gtest.h>


#include <ginkgo/core/base/executor.hpp>
#include <ginkgo/core/base/math.hpp>
#include <ginkgo/core/matrix/dense.hpp>
#include <ginkgo/core/stop/iteration.hpp>


#include "core/test/utils.hpp"


namespace {


template <typename T>
class Convergence : public ::testing::Test {};

TYPED_TEST_SUITE(Convergence, gko::test::ValueTypes, TypenameNameGenerator);


TYPED_TEST(Convergence, CatchesCriterionCheckCompleted)
{
    auto exec = gko::ReferenceExecutor::create();
    auto logger = gko::log::Convergence<TypeParam>::create(
        gko::log::Logger::criterion_check_completed_mask);
    auto criterion =
        gko::stop::Iteration::build().with_max_iters(3u).on(exec)->generate(
            nullptr, nullptr, nullptr);
    constexpr gko::uint8 RelativeStoppingId{42};
    gko::array<gko::stopping_status> stop_status(exec, 1);
    stop_status.get_data()[0].reset();
    using Mtx = gko::matrix::Dense<TypeParam>;
    using NormVector = gko::matrix::Dense<gko::remove_complex<TypeParam>>;
    auto residual = gko::initialize<Mtx>({1.0, 2.0, 2.0}, exec);

    logger->template on<gko::log::Logger::criterion_check_completed>(
        criterion.get(), 1, residual.get(), nullptr, nullptr,
        RelativeStoppingId, true, &stop_status, true, true);

    ASSERT_EQ(logger->get_num_iterations(), 1);
    ASSERT_EQ(logger->has_converged(), false);
    GKO_ASSERT_MTX_NEAR(gko::as<Mtx>(logger->get_residual()),
                        l({1.0, 2.0, 2.0}), 0.0);
    GKO_ASSERT_MTX_NEAR(gko::as<NormVector>(logger->get_residual_norm()),
                        l({3.0}), 0.0);
}


TYPED_TEST(Convergence, CatchesCriterionCheckCompletedWithConvCheck)
{
    auto exec = gko::ReferenceExecutor::create();
    auto logger = gko::log::Convergence<TypeParam>::create(
        gko::log::Logger::criterion_check_completed_mask);
    auto criterion =
        gko::stop::Iteration::build().with_max_iters(3u).on(exec)->generate(
            nullptr, nullptr, nullptr);
    constexpr gko::uint8 RelativeStoppingId{42};
    gko::array<gko::stopping_status> stop_status(exec, 1);
    stop_status.get_data()[0].reset();
    stop_status.get_data()[0].converge(0);
    using Mtx = gko::matrix::Dense<TypeParam>;
    using NormVector = gko::matrix::Dense<gko::remove_complex<TypeParam>>;
    auto residual = gko::initialize<Mtx>({1.0, 2.0, 2.0}, exec);

    logger->template on<gko::log::Logger::criterion_check_completed>(
        criterion.get(), 3, residual.get(), nullptr, nullptr,
        RelativeStoppingId, true, &stop_status, true, true);

    ASSERT_EQ(logger->get_num_iterations(), 3);
    ASSERT_EQ(logger->has_converged(), true);
    GKO_ASSERT_MTX_NEAR(gko::as<Mtx>(logger->get_residual()),
                        l({1.0, 2.0, 2.0}), 0.0);
    GKO_ASSERT_MTX_NEAR(gko::as<NormVector>(logger->get_residual_norm()),
                        l({3.0}), 0.0);
}


TYPED_TEST(Convergence, CatchesCriterionCheckCompletedWithStopCheck)
{
    auto exec = gko::ReferenceExecutor::create();
    auto logger = gko::log::Convergence<TypeParam>::create(
        gko::log::Logger::criterion_check_completed_mask);
    auto criterion =
        gko::stop::Iteration::build().with_max_iters(3u).on(exec)->generate(
            nullptr, nullptr, nullptr);
    constexpr gko::uint8 RelativeStoppingId{42};
    gko::array<gko::stopping_status> stop_status(exec, 1);
    stop_status.get_data()[0].reset();
    stop_status.get_data()[0].stop(0);
    using Mtx = gko::matrix::Dense<TypeParam>;
    using NormVector = gko::matrix::Dense<gko::remove_complex<TypeParam>>;
    auto residual = gko::initialize<Mtx>({1.0, 2.0, 2.0}, exec);

    logger->template on<gko::log::Logger::criterion_check_completed>(
        criterion.get(), 3, residual.get(), nullptr, nullptr,
        RelativeStoppingId, true, &stop_status, true, true);

    ASSERT_EQ(logger->get_num_iterations(), 3);
    ASSERT_EQ(logger->has_converged(), false);
    GKO_ASSERT_MTX_NEAR(gko::as<Mtx>(logger->get_residual()),
                        l({1.0, 2.0, 2.0}), 0.0);
    GKO_ASSERT_MTX_NEAR(gko::as<NormVector>(logger->get_residual_norm()),
                        l({3.0}), 0.0);
}


TYPED_TEST(Convergence, CanResetConvergenceStatus)
{
    auto exec = gko::ReferenceExecutor::create();
    auto logger = gko::log::Convergence<TypeParam>::create(
        gko::log::Logger::criterion_check_completed_mask);
    auto criterion =
        gko::stop::Iteration::build().with_max_iters(3u).on(exec)->generate(
            nullptr, nullptr, nullptr);
    constexpr gko::uint8 RelativeStoppingId{42};
    gko::array<gko::stopping_status> stop_status(exec, 1);
    stop_status.get_data()[0].reset();
    stop_status.get_data()[0].converge(0);
    using Mtx = gko::matrix::Dense<TypeParam>;
    using NormVector = gko::matrix::Dense<gko::remove_complex<TypeParam>>;
    auto residual = gko::initialize<Mtx>({1.0, 2.0, 2.0}, exec);

    logger->template on<gko::log::Logger::criterion_check_completed>(
        criterion.get(), 3, residual.get(), nullptr, nullptr,
        RelativeStoppingId, true, &stop_status, true, true);
    ASSERT_EQ(logger->get_num_iterations(), 3);
    ASSERT_EQ(logger->has_converged(), true);

    logger->reset_convergence_status();

    ASSERT_EQ(logger->has_converged(), false);
}


TYPED_TEST(Convergence, CatchesCriterionCheckCompletedWithImplicitNorm)
{
    auto exec = gko::ReferenceExecutor::create();
    auto logger = gko::log::Convergence<TypeParam>::create(
        gko::log::Logger::criterion_check_completed_mask);
    auto criterion =
        gko::stop::Iteration::build().with_max_iters(3u).on(exec)->generate(
            nullptr, nullptr, nullptr);
    constexpr gko::uint8 RelativeStoppingId{42};
    gko::array<gko::stopping_status> stop_status(exec, 1);
    stop_status.get_data()[0].reset();
    using Mtx = gko::matrix::Dense<TypeParam>;
    using NormVector = gko::matrix::Dense<gko::remove_complex<TypeParam>>;
    auto residual = gko::initialize<Mtx>({1.0, 2.0, 2.0}, exec);
    auto implicit_sq_resnorm = gko::initialize<Mtx>({4.0}, exec);

    logger->template on<gko::log::Logger::criterion_check_completed>(
        criterion.get(), 1, residual.get(), nullptr, implicit_sq_resnorm.get(),
        nullptr, RelativeStoppingId, true, &stop_status, true, true);

    ASSERT_EQ(logger->get_num_iterations(), 1);
    ASSERT_EQ(logger->has_converged(), false);
    GKO_ASSERT_MTX_NEAR(gko::as<Mtx>(logger->get_implicit_sq_resnorm()),
                        l({4.0}), 0.0);
    GKO_ASSERT_MTX_NEAR(gko::as<Mtx>(logger->get_residual()),
                        l({1.0, 2.0, 2.0}), 0.0);
    GKO_ASSERT_MTX_NEAR(gko::as<NormVector>(logger->get_residual_norm()),
                        l({3.0}), 0.0);
}


}  // namespace
